package model.fish.bad;

import model.ILambda;
import model.fish.display.ImageFishDisplay;
import sysModel.fish.Fish;
import sysModel.fish.IFishBehavior;

import java.awt.*;

/**
 * A fish that produces a stack overflow.
 *
 * @author Mathias Ricken
 */
public class StackOverflowFish extends Fish {
    /**
     * Constructor.
     *
     * @param fishColor color of the fish
     */
    public StackOverflowFish(Color fishColor) {
        super(fishColor, new ImageFishDisplay("smallfish.gif", 3.0 * Math.PI / 2.0), new IFishBehavior() {
            /**
             * Carry out movement behavior for the host fish.
             *
             * @param host fish to move.
             */
            public void move(final Fish host) {
                final ILambda lambda[] = new ILambda[2];
                lambda[0] = new ILambda() {
                    public Object apply(Object param) {
                        // cannot move, just retry
                        host.tryMoveFwd(lambda[0], lambda[1]);

                        return null;
                    }
                };
                lambda[1] = new ILambda() {
                    public Object apply(Object param) {
                        // can move forward, but retry
                        host.tryMoveFwd(lambda[0], lambda[1]);

                        return null;
                    }
                };

                // attempt to move forward
                host.tryMoveFwd(lambda[0], lambda[1]);
            }
        });
    }
}
