package model.fish.behavior;

import lrs.LRStruct;
import lrs.visitor.GetLength;
import lrs.visitor.GetNth;
import model.ILambda;
import model.RandNumGenerator;
import sysModel.fish.Fish;
import sysModel.fish.IFishBehavior;

/**
 * Class for a generic fish.
 *
 * @author Mathias G. Ricken
 */
public class GenericBehavior implements IFishBehavior {
    /**
     * Carry out movement behavior for the host fish.
     *
     * @param host fish to move.
     */
    public void move(final Fish host) {
        final LRStruct moveLambdas = new LRStruct();

        // turn PI/2 radians to the left and attempt to move forward
        host.turnLeft();

        final ILambda blocked3 = new ILambda() {
            public Object apply(Object param) {
                int size = ((Integer)moveLambdas.execute(GetLength.Singleton, null)).intValue();

                if (size != 0) {
                    // pick random move command
                    Integer randnum = new Integer(RandNumGenerator.instance().nextInt(size));
                    ILambda cmd = (ILambda)moveLambdas.execute(GetNth.Singleton, randnum);

                    // apply that command
                    cmd.apply(null);
                }
                else {
                    // turn back PI/2 radians to the left
                    host.turnLeft();
                }
                return null;
            }
        };

        final ILambda blocked2 = new ILambda() {
            public Object apply(Object param) {
                // turn PI/2 radians to the right and attempt to move forward
                host.turnRight();
                return host.tryMoveFwd(blocked3,
                    new ILambda() {
                        public Object apply(Object param) {
                            // the field to the right is open
                            moveLambdas.insertFront(param);
                            return blocked3.apply(null);
                        }
                    });
            }
        };

        final ILambda blocked1 = new ILambda() {
            public Object apply(Object param) {
                // turn PI/2 radians to the right and attempt to move forward
                host.turnRight();
                return host.tryMoveFwd(blocked2,
                    new ILambda() {
                        public Object apply(Object param) {
                            // the field straight ahead is open
                            moveLambdas.insertFront(param);
                            return blocked2.apply(null);
                        }
                    });
            }
        };


        host.tryMoveFwd(blocked1,
            new ILambda() {
                public Object apply(Object param) {
                    // the field to the left is open
                    moveLambdas.insertFront(param);
                    return blocked1.apply(null);
                }
            });
    }

    /**
     * Private singleton ctor.
     */
    private GenericBehavior() {
    }

    /**
     * Singleton instance.
     */
    public static final GenericBehavior Singleton = new GenericBehavior();

}
