package model.fish.behavior;

import model.RandNumGenerator;
import sysModel.fish.Fish;
import sysModel.fish.IFishBehavior;

/**
 * Class for a slow fish.
 *
 * @author Mathias G. Ricken
 */
public class SlowBehavior implements IFishBehavior {
    /**
     * Probability that this slow fish moves.
     */
    private double _probOfMoving = 1.0f / 5.0f;

    /**
     * Carry out movement behavior for the host fish.
     *
     * @param host fish to move.
     */
    public void move(Fish host) {
        if (RandNumGenerator.instance().nextDouble() < _probOfMoving) {
            // slow fish attempts to move like super class
            GenericBehavior.Singleton.move(host);
        }
    }

    /**
     * Private singleton ctor.
     */
    private SlowBehavior() {
    }

    /**
     * Singleton instance.
     */
    public static final SlowBehavior Singleton = new SlowBehavior();

}
